==> gm-1.6.3_Linux.tar.gz: gm-1.6.3_Linux/drivers/aix/methods/ucfggmpci.c <==

/*
 * PRODUCT: gmpci - PCI WatchDog Timer Device Driver
 *
 * CONTENTS: Unconfig Method
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/cfgdb.h>
#include <sys/cfgodm.h>
#include <cf.h>
#include <sys/sysconfig.h>
#include <sys/sysmacros.h>
#include <sys/device.h>
#include <sys/stat.h>
#include <sys/errno.h>
#include "gm_dds.h"


#ifdef DEBUG
#define DBG(x) x
#else
#define DBG(x)
#endif

/*==========================================================================
 * Local prototypes
 *==========================================================================*/
void err_exit(int exitcode);

/*==========================================================================
 * Externals
 *==========================================================================*/

static char *prog;

#ifdef DEBUG
static FILE *dbg_fp;
#endif

/*===========================================================================
 *
 * NAME:	main
 *
 * FUNCTION:	Unconfigure method for PCI WatchDog Timer adapter
 *
 * SYNTAX:	ucfggmpci -l <logical_name>
 *
 * EXIT CODE:   from cf.h: E_OK if successful, otherwise specific error code
 *
 *===========================================================================*/

main(
int argc,
char *argv[]
)
{
	struct cfg_load cfg_load;	/* sysconfig load structure */
	struct cfg_kmod cfg_kmod;	/* sysconfig call kmod structure */

	char *lname = NULL;          	/* Logical name to unconfigure */
	char criteria[256];		/* Search criteria string */
	char *odmdir_env;		/* Environment ODM directory */
	char lock_file[128];		/* Name of lock file */

	struct CuDv CuDv;		/* CuDv for this device */
	struct PdDv PdDv;		/* PdDv for this device */

	struct Class *cusdev;		/* CuDv class handle */

	int rc;				/* Return code */
	int errflg = 0;             	/* Used in parsing parameters   */
	char drvrpath[PATH_MAX+1];	/* Buffer for full driver pathname */
	int c;               		/* Used in parsing parameters   */

	time_t now;			/* Updated by time() */

	extern int optind;		/* Defined by getopt() */
	extern char *optarg;		/* Defined by getopt() */

	dds_t	gmpci_dds;

#ifdef DEBUG
	time(&now);
#ifdef DBG_FILE
 	dbg_fp = fopen(DBG_FILE, "a+");
#else
	dbg_fp = stderr;	/* Don't freopen because that affects all */
#endif
#endif

	DBG(fprintf(dbg_fp, "\n%s: main %s", argv[0], ctime(&now)));
	prog = argv[0];

	/* Get options
	 */
	while ((c = getopt(argc,argv,"l:")) != EOF) {
		switch (c) {
		case 'l':
			if (lname != NULL)
				errflg++;
			lname = optarg;
			break;
		default:
			errflg++;
		}
	}

	/* If error getting options, exit
	 */
	if (errflg) {
		DBG(fprintf(dbg_fp, "%s: command line error\n", argv[0]));
		exit(E_ARGS);
	}

	/* Validate command line options: if logical name not specified, exit
	 */
	if (lname == NULL) {
		DBG(fprintf(dbg_fp, "%s: logical name must be specified\n",
			    					      argv[0]));
		exit(E_LNAME);
	}

	DBG(fprintf(dbg_fp, "%s: Unconfiguring: %s\n", argv[0], lname));

	/* Start up odm
	 */
	if (odm_initialize() == -1) {
		DBG(fprintf(dbg_fp, "%s: odm_initialize() failed\n", argv[0]));
		exit(E_ODMINIT);
	}

	/* Lock the database
	 */
	if ((odmdir_env = getenv("ODMDIR")) == NULL)
		sprintf(lock_file, "/etc/objrepos/config_lock");
	else
		sprintf(lock_file, "%s/%s", odmdir_env, "config_lock");

	if (odm_lock(lock_file, 60) == -1) {
		DBG(fprintf(dbg_fp, "%s: odm_lock() failed\n", argv[0]));
		err_exit(E_ODMLOCK);
	}
	DBG(fprintf(dbg_fp, "%s: ODM initialized and locked\n", argv[0]));

	/* Open CuDv object class
	 */
	if ((int)(cusdev=odm_open_class(CuDv_CLASS)) == -1) {
		DBG(fprintf(dbg_fp, "%s: open of CuDv failed\n", argv[0]));
		err_exit(E_ODMOPEN);
	}

	/* Get device's CuDv object
	 */
	sprintf(criteria, "name = '%s'", lname);
	rc = (int)odm_get_first(cusdev, criteria, &CuDv);
	if (rc == 0) {
		DBG(fprintf(dbg_fp, "%s: Device does not exist\n", argv[0]));
		err_exit(E_NOCuDv);
	}
	if (rc== -1) {
		DBG(fprintf(dbg_fp, "%s: ODM error getting CuDv object\n",
			    					      argv[0]));
		err_exit(E_ODMGET);
	}

	/* If device state is DEFINED, then already unconfigured
	 * so just exit
	 */
	if (CuDv.status == DEFINED) {
		DBG(fprintf(dbg_fp, "%s: device is already DEFINED state",
			    					      argv[0]));
		err_exit(E_OK);
	}

	/* Get device's PdDv object
	 */
	sprintf(criteria, "uniquetype = '%s'", CuDv.PdDvLn_Lvalue);
	rc = (int)odm_get_first(PdDv_CLASS, criteria, &PdDv);
	if (rc == 0) {
		DBG(fprintf(dbg_fp, "%s: No PdDv object\n", argv[0]));
		err_exit(E_NOPdDv);
	}
	if (rc == -1) {
		DBG(fprintf(dbg_fp, "%s: ODM error getting PdDv object\n",
			    					    argv[0]));
		err_exit(E_ODMGET);
	}

	/* If device has a driver, terminate the device and unload driver
	 */
	if (strcmp(PdDv.DvDr, "") != 0) {
		DBG(fprintf(dbg_fp,  "%s: Device has driver\n", argv[0]));

		/* Query to determine the kmid of the loaded device driver.
		 * Should have been loaded from /usr/lib/drivers.
		 * If error querying, exit.
		 */
#define STD_DRVR_DIR "/usr/lib/drivers/"

		strcpy(drvrpath, STD_DRVR_DIR);
		strcat(drvrpath, PdDv.DvDr);
		cfg_load.path = drvrpath;
		cfg_load.libpath = NULL;
		cfg_load.kmid = NULL;
		DBG(fprintf(dbg_fp, "%s: Querying driver %s via sysconfig()\n",
			    			     argv[0], cfg_load.path));
		if ((rc = sysconfig(SYS_QUERYLOAD, &cfg_load,
					    	   sizeof cfg_load)) == -1) {
			DBG(fprintf(dbg_fp, "%s: error (%d) from sysconfig query load\n",
					    		      argv[0], errno));
			err_exit(E_SYSCONFIG);
		}
		
		/* If not found, exit
		 */
		if (cfg_load.kmid == NULL) {
			DBG(fprintf(dbg_fp, "%s: %s driver not loaded\n",
					    	     argv[0], cfg_load.path));
			err_exit(E_SYSCONFIG);
		}

		/* Call the driver's config entry point with CFG_TERM
		 */
		DBG(fprintf(dbg_fp, "%s: Terminating driver via sysconfig()\n",
				    				     argv[0]));
		/* Setup gmpci_dds
		 */
		gmpci_dds.seq_number = atoi(lname + strlen(PdDv.prefix));

		cfg_kmod.kmid = cfg_load.kmid;
		cfg_kmod.cmd = CFG_TERM;
		cfg_kmod.mdiptr = (uchar*)&gmpci_dds;
		cfg_kmod.mdilen = sizeof gmpci_dds;

		if ((rc = sysconfig(SYS_CFGKMOD, &cfg_kmod, sizeof cfg_kmod))
		    						      == -1) {
			/* If call fails with EBUSY (device is open), exit
			 * with error.  Otherwise ignore failure since the
			 * driver will terminate the device even if it reports
			 * some other error.
			 */
			if (errno == EBUSY) {
				DBG(fprintf(dbg_fp, "%s: Device is busy\n",
					    			     argv[0]));
				err_exit(E_BUSY);
			}
		}
		DBG(fprintf(dbg_fp, "%s: Called sysconfig() to terminate device\n",
			    					    argv[0]));

		/* Unload driver
		 */
		if (sysconfig(SYS_KULOAD, &cfg_load, sizeof cfg_load) == -1) {
			DBG(fprintf(dbg_fp, "%s: error unloading driver\n", prog));
		}
		else
			DBG(fprintf(dbg_fp, "%s: Unloaded driver\n", argv[0]));

		if (reldevno (lname, TRUE) == -1) {
			DBG(fprintf(dbg_fp, "%s: error releasing majorno\n",
				argv[0]));
		}
	}

	/* Change the status field of device to DEFINED
	 */
	DBG(fprintf(dbg_fp, "%s: Updating device state\n", argv[0]));
	CuDv.status = DEFINED;
	if (odm_change_obj(cusdev, &CuDv) == -1) {
		DBG(fprintf(dbg_fp, "%s: update of CuDv object failed",
			    					    argv[0]));
		err_exit(E_ODMUPDATE);
	}

	/* Close the CuDv object class
	 */
	if (odm_close_class(CuDv_CLASS) == -1) {
		DBG(fprintf(dbg_fp, "%s: error closing CuDv object class\n",
			    					    argv[0]));
		err_exit(E_ODMCLOSE);
	}

	odm_terminate();

	DBG(fprintf(dbg_fp, "%s: successful exit\n", argv[0]));
	exit(E_OK);
}

/*===========================================================================
 *
 * NAME:	err_exit
 *
 * FUNCTION:	Closes any open object classes and terminates ODM.  Used to
 *           	back out of an error.  Does not return.
 *
 * ARGUMENTS:	Exit code
 *
 * EXIT CODE:   from cf.h: E_OK if successful, otherwise specific error code
 *
 *===========================================================================*/

void
err_exit(
int exitcode
)
{
	odm_close_class(CuDv_CLASS);
	odm_terminate();
	exit(exitcode);
}

==> gm-2.0.8_Linux.tar.gz: gm-2.0.8_Linux/drivers/aix/methods/ucfggmpci.c <==

/*
 * PRODUCT: gmpci - Myrinet PCI Device Driver
 *
 * CONTENTS: Unconfig Method
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/cfgdb.h>
#include <sys/cfgodm.h>
#include <cf.h>
#include <sys/sysconfig.h>
#include <sys/sysmacros.h>
#include <sys/device.h>
#include <sys/stat.h>
#include <sys/errno.h>
#include "gm_dds.h"


/*==========================================================================
 * To get debugging output, modify CFLAGS in ./Makefile to define DEBUG and
 * (optionally) DBG_FILE.  The output will be written to the file identified
 * by DBG_FILE, else to stderr.
 *==========================================================================*/
   
#ifdef DEBUG
#define DBG(x) x
#else
#define DBG(x)
#endif

/*==========================================================================
 * Local prototypes
 *==========================================================================*/
void err_exit(int exitcode);

/*==========================================================================
 * Externals
 *==========================================================================*/

static char *prog;

#ifdef DEBUG
static FILE *dbg_fp;
#endif

/*===========================================================================
 *
 * NAME:	main
 *
 * FUNCTION:	Unconfigure method for Myrinet PCI adapter
 *
 * SYNTAX:	ucfggmpci -l <logical_name>
 *
 * EXIT CODE:   from cf.h: E_OK if successful, otherwise specific error code
 *
 *===========================================================================*/

main(
int argc,
char *argv[]
)
{
	struct cfg_load cfg_load;	/* sysconfig load structure */
	struct cfg_kmod cfg_kmod;	/* sysconfig call kmod structure */

	char *lname = NULL;          	/* Logical name to unconfigure */
	char criteria[256];		/* Search criteria string */
	char *odmdir_env;		/* Environment ODM directory */
	char lock_file[128];		/* Name of lock file */

	struct CuDv CuDv;		/* CuDv for this device */
	struct PdDv PdDv;		/* PdDv for this device */

	struct Class *cusdev;		/* CuDv class handle */

	int rc;				/* Return code */
	int errflg = 0;             	/* Used in parsing parameters   */
	char drvrpath[PATH_MAX+1];	/* Buffer for full driver pathname */
	int c;               		/* Used in parsing parameters   */

	time_t now;			/* Updated by time() */

	extern int optind;		/* Defined by getopt() */
	extern char *optarg;		/* Defined by getopt() */

	dds_t	gmpci_dds;

#ifdef DEBUG
	time(&now);
#ifdef DBG_FILE
 	dbg_fp = fopen(DBG_FILE, "a+");
	if (dbg_fp == NULL) {
	  fprintf(stderr, "open of %s failed with error %d; using stderr\n",
		  DBG_FILE, errno);
	  dbg_fp = stderr;	/* Don't freopen because that affects all */
	}
#else
	dbg_fp = stderr;	/* Don't freopen because that affects all */
#endif
#endif

	DBG(fprintf(dbg_fp, "\n%s: main %s", argv[0], ctime(&now)));
	prog = argv[0];

	/* Get options
	 */
	while ((c = getopt(argc,argv,"l:")) != EOF) {
		switch (c) {
		case 'l':
			if (lname != NULL)
				errflg++;
			lname = optarg;
			break;
		default:
			errflg++;
		}
	}

	/* If error getting options, exit
	 */
	if (errflg) {
		fprintf(stderr, "%s: command line error\n", argv[0]);
		exit(E_ARGS);
	}

	/* Validate command line options: if logical name not specified, exit
	 */
	if (lname == NULL) {
		fprintf(stderr,
			"%s: logical name must be specified\n", argv[0]);
		exit(E_LNAME);
	}

	DBG(fprintf(dbg_fp, "%s: Unconfiguring: %s\n", argv[0], lname));

	/* Start up odm
	 */
	if (odm_initialize() == -1) {
		fprintf(stderr, "%s: odm_initialize() failed\n", argv[0]);
		exit(E_ODMINIT);
	}

	/* Lock the database
	 */
	if ((odmdir_env = getenv("ODMDIR")) == NULL)
		sprintf(lock_file, "/etc/objrepos/config_lock");
	else
		sprintf(lock_file, "%s/%s", odmdir_env, "config_lock");

	if (odm_lock(lock_file, 60) == -1) {
		fprintf(stderr, "%s: odm_lock() failed\n", argv[0]);
		err_exit(E_ODMLOCK);
	}
	DBG(fprintf(dbg_fp, "%s: ODM initialized and locked\n", argv[0]));

	/* Open CuDv object class
	 */
	if ((int)(cusdev=odm_open_class(CuDv_CLASS)) == -1) {
		fprintf(stderr, "%s: open of CuDv failed\n", argv[0]);
		err_exit(E_ODMOPEN);
	}

	/* Get device's CuDv object
	 */
	sprintf(criteria, "name = '%s'", lname);
	rc = (int)odm_get_first(cusdev, criteria, &CuDv);
	if (rc == 0) {
		fprintf(stderr, "%s: Device does not exist\n", argv[0]);
		err_exit(E_NOCuDv);
	}
	if (rc== -1) {
		fprintf(stderr,
			"%s: ODM error getting CuDv object\n", argv[0]);
		err_exit(E_ODMGET);
	}

	/* If device state is DEFINED, then already unconfigured
	 * so just exit
	 */
	if (CuDv.status == DEFINED) {
		fprintf(stderr,
			"%s: device is already DEFINED state", argv[0]);
		err_exit(E_OK);
	}

	/* Get device's PdDv object
	 */
	sprintf(criteria, "uniquetype = '%s'", CuDv.PdDvLn_Lvalue);
	rc = (int)odm_get_first(PdDv_CLASS, criteria, &PdDv);
	if (rc == 0) {
		fprintf(stderr, "%s: No PdDv object\n", argv[0]);
		err_exit(E_NOPdDv);
	}
	if (rc == -1) {
		fprintf(stderr,
			"%s: ODM error getting PdDv object\n", argv[0]);
		err_exit(E_ODMGET);
	}

	/* If device has a driver, terminate the device and unload driver
	 */
	if (strcmp(PdDv.DvDr, "") != 0) {
		DBG(fprintf(dbg_fp,  "%s: Device has driver\n", argv[0]));

		/* Query to determine the kmid of the loaded device driver.
		 * Should have been loaded from /usr/lib/drivers.
		 * If error querying, exit.
		 */
#define STD_DRVR_DIR "/usr/lib/drivers/"

		strcpy(drvrpath, STD_DRVR_DIR);
		strcat(drvrpath, PdDv.DvDr);
		cfg_load.path = drvrpath;
		cfg_load.libpath = NULL;
		cfg_load.kmid = NULL;
		DBG(fprintf(dbg_fp, "%s: Querying driver %s via sysconfig()\n",
			    			     argv[0], cfg_load.path));
		if ((rc = sysconfig(SYS_QUERYLOAD, &cfg_load,
					    	   sizeof cfg_load)) == -1) {
			fprintf(stderr,
				"%s: error (%d) from sysconfig query load\n",
			       argv[0], errno);
			err_exit(E_SYSCONFIG);
		}
		
		/* If not found, exit
		 */
		if (cfg_load.kmid == NULL) {
			fprintf(stderr, "%s: %s driver not loaded\n",
				argv[0], cfg_load.path);
			err_exit(E_SYSCONFIG);
		}

		/* Call the driver's config entry point with CFG_TERM
		 */
		DBG(fprintf(dbg_fp, "%s: Terminating driver via sysconfig()\n",
				    				     argv[0]));
		DBG(fflush(dbg_fp));
		/* Setup gmpci_dds
		 */
		gmpci_dds.seq_number = atoi(lname + strlen(PdDv.prefix));

		cfg_kmod.kmid = cfg_load.kmid;
		cfg_kmod.cmd = CFG_TERM;
		cfg_kmod.mdiptr = (uchar*)&gmpci_dds;
		cfg_kmod.mdilen = sizeof gmpci_dds;

		if ((rc = sysconfig(SYS_CFGKMOD, &cfg_kmod, sizeof cfg_kmod))
		    						      == -1) {
			/* If call fails with EBUSY (device is open), exit
			 * with error.  Otherwise ignore failure since the
			 * driver will terminate the device even if it reports
			 * some other error.
			 */
			if (errno == EBUSY) {
				fprintf(stderr,
					"%s: Device is busy\n", argv[0]);
				err_exit(E_BUSY);
			}
		}
		DBG(fprintf(dbg_fp, "%s: Called sysconfig() to terminate device\n",
			    					    argv[0]));
		DBG(fflush(dbg_fp));

		/* Unload driver
		 */
		sleep(3);      /* give other threads a chance to exit open() */

		if (sysconfig(SYS_KULOAD, &cfg_load, sizeof cfg_load) == -1) {
			fprintf(stderr, "%s: error unloading driver\n", prog);
		}
		else {
			DBG(fprintf(dbg_fp, "%s: Unloaded driver\n", argv[0]));
			DBG(fflush(dbg_fp));
		}
		if (reldevno (lname, TRUE) == -1) {
			fprintf(stderr,
				"%s: error releasing majorno\n", argv[0]);
		}
	}

	/* Change the status field of device to DEFINED
	 */
	DBG(fprintf(dbg_fp, "%s: Updating device state\n", argv[0]));
	CuDv.status = DEFINED;
	if (odm_change_obj(cusdev, &CuDv) == -1) {
		fprintf(stderr, "%s: update of CuDv object failed", argv[0]);
		err_exit(E_ODMUPDATE);
	}

	/* Close the CuDv object class
	 */
	if (odm_close_class(CuDv_CLASS) == -1) {
		fprintf(stderr,
			"%s: error closing CuDv object class\n", argv[0]);
		err_exit(E_ODMCLOSE);
	}

	odm_terminate();

	DBG(fprintf(dbg_fp, "%s: successful exit\n", argv[0]));
	exit(E_OK);
}

/*===========================================================================
 *
 * NAME:	err_exit
 *
 * FUNCTION:	Closes any open object classes and terminates ODM.  Used to
 *           	back out of an error.  Does not return.
 *
 * ARGUMENTS:	Exit code
 *
 * EXIT CODE:   from cf.h: E_OK if successful, otherwise specific error code
 *
 *===========================================================================*/

void
err_exit(
int exitcode
)
{
	odm_close_class(CuDv_CLASS);
	odm_terminate();
	exit(exitcode);
}
